package com.xy.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.xy.dto.DeviceQualityValueDto;
import com.xy.entity.DeviceQualityValue;
import com.xy.mapper.DeviceQualityValueMapper;
import com.xy.utils.*;
import com.xy.utils.enums.QualityEnum;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import java.time.LocalDateTime;
import java.util.List;

import static com.xy.utils.Beans.copy;


/**
 * <p>
 * 设备-质检值 服务实现类
 * </p>
 *
 * @author lijin
 * @since 2023-01-30
 */
@Service
@AllArgsConstructor
@Api(tags = "设备-质检值")
public class DeviceQualityValueServiceImpl extends ServiceImpl<DeviceQualityValueMapper, DeviceQualityValue> implements DeviceQualityValueService {

    @PostMapping("obj")
    @ApiOperation("对象查询")
    public R<DeviceQualityValueDto.Vo> obj(@RequestBody DeviceQualityValueDto.Vo vo) {
        LambdaQueryWrapper<DeviceQualityValue> lambdaQueryWrapper = new MybatisPlusQuery().eqWrapper(vo, DeviceQualityValue.class).build();
        List<DeviceQualityValue> list = list(lambdaQueryWrapper);
        if (Emptys.check(list)) {
            return R.ok(copy(DeviceQualityValueDto.Vo.class, list.get(0)));
        }
        return R.ok();
    }

    @PostMapping("save")
    @ApiOperation("添加")
    public R save(@RequestBody List<DeviceQualityValueDto.Vo> vos) {
        LocalDateTime now = LocalDateTime.now();
        List<DeviceQualityValue> list = copy(DeviceQualityValue.class, vos);
        list.forEach(deviceQualityValue -> deviceQualityValue.setCreateTime(now).setUpdateTime(now));
        saveBatch(list);
        return R.ok();
    }

    @PostMapping("update")
    @ApiOperation("修改")
    public R update(@RequestBody DeviceQualityValueDto.Update update) {
        if (Emptys.check(update.getCode())) {
            UpdateWrapper<DeviceQualityValue> updateWrapper = new UpdateWrapper<DeviceQualityValue>()
                    .set(QualityEnum.getQualityEnum(update.getCode()).getValue(), update.getValue() == null ? null : update.getValue().toString())
                    .set(StringTools.humpToLine(LambdaUtils.getProperty(DeviceQualityValue::getUpdateTime)), LocalDateTime.now())
                    .eq(StringTools.humpToLine(LambdaUtils.getProperty(DeviceQualityValue::getQualityId)), update.getQualityId());
            update(updateWrapper);
        } else {
            DeviceQualityValue deviceQualityValue = copy(DeviceQualityValue.class, update)
                    .setUpdateTime(LocalDateTime.now());
            updateById(deviceQualityValue);
        }
        return R.ok();
    }

}