package com.xy.dto;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.xy.utils.DataTime;
import com.xy.utils.PageBean;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.experimental.Accessors;

import javax.validation.constraints.NotNull;
import java.time.LocalDate;
import java.time.LocalDateTime;

@Data
@Accessors(chain = true)
public class DeviceInfoDto {

    @Data
    @Accessors(chain = true)
    public static class Obj {

        @ApiModelProperty("设备id")
        private Long deviceId;
    }

    @Data
    @Accessors(chain = true)
    public static class Update extends Vo {

        @NotNull(message = "deviceId不能为空")
        @ApiModelProperty("设备id")
        private Long deviceId;

        @ApiModelProperty(value = "系统信息", hidden = true)
        private DeviceSysinfoDto.Vo deviceSysinfo;

        @ApiModelProperty(value = "状态信息", hidden = true)
        private DeviceStatusDto.Vo deviceStatus;

        @ApiModelProperty(value = "注册信息", hidden = true)
        private DeviceRegisterDto.Vo deviceRegister;
    }

    @Data
    @Accessors(chain = true)
    public static class Page extends Vo {

        @ApiModelProperty(value = "分页对象", required = true)
        private PageBean page;

        @ApiModelProperty("机器/资产/sn/sim号")
        private String no;

        @ApiModelProperty("联网状态")
        private Integer netState;

        @ApiModelProperty("商户号或名称")
        private String merc;

        @ApiModelProperty("上位机版本号")
        private String appUpmVersion;

        @ApiModelProperty("下位机版本号")
        private String appDownmVersion;

        @JsonFormat(pattern = "yyyy-MM-dd")
        @ApiModelProperty(value = "激活时间-起")
        private LocalDate beginActiveTime;

        @JsonFormat(pattern = "yyyy-MM-dd")
        @ApiModelProperty(value = "激活时间-始")
        private LocalDate endActiveTime;

        @ApiModelProperty("附近最大距离，单位：米")
        private Integer nearby;

        public String getBeginActiveTime() {
            return beginActiveTime == null ? null : DataTime.toString(beginActiveTime.atTime(0, 0, 0));
        }

        public String getEndActiveTime() {
            return endActiveTime == null ? null : DataTime.toString(endActiveTime.atTime(23, 59, 59));
        }

        @ApiModelProperty(value = "系统信息", hidden = true)
        private DeviceSysinfoDto.Vo deviceSysinfo;

        @ApiModelProperty(value = "状态信息", hidden = true)
        private DeviceStatusDto.Vo deviceStatus;

        @ApiModelProperty(value = "注册信息", hidden = true)
        private DeviceRegisterDto.Vo deviceRegister;
    }

    @Data
    @Accessors(chain = true)
    public static class Vo {

        @ApiModelProperty("设备id")
        private Long deviceId;

        @ApiModelProperty("设备类型")
        private Integer deviceType;

        @ApiModelProperty("商户设备资产编号")
        private String mercDeviceCode;

        @ApiModelProperty("商户id")
        private Long mercId;

        @ApiModelProperty("商户编码")
        private String mercCode;

        @ApiModelProperty("点位id")
        private Long placeId;

        @ApiModelProperty("线路id")
        private Long placeLineId;

        @ApiModelProperty("区域id")
        private Long districtId;

        @ApiModelProperty("激活状态")
        private Integer activeState;

        @ApiModelProperty("激活时间")
        @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
        private LocalDateTime activeTime;

        @ApiModelProperty("运营状态")
        private Integer busyState;

        @ApiModelProperty("显示状态")
        private Integer showStatus;

        @ApiModelProperty("设备图片")
        private String deviceImg;

        @ApiModelProperty("经度")
        private String lon;

        @ApiModelProperty("纬度")
        private String lat;

        @ApiModelProperty("创建时间")
        @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
        private LocalDateTime createTime;

        @ApiModelProperty("更新时间")
        @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
        private LocalDateTime updateTime;

        @ApiModelProperty("系统信息")
        private DeviceSysinfoDto.Vo deviceSysinfo;

        @ApiModelProperty("状态信息")
        private DeviceStatusDto.Vo deviceStatus;

        @ApiModelProperty("注册信息")
        private DeviceRegisterDto.Vo deviceRegister;
    }

    @Data
    @Accessors(chain = true)
    public static class Vo2 extends Vo {

        @ApiModelProperty("安卓版本号")
        private String androidVersion;

        @ApiModelProperty("sn号")
        private String deviceSn;

        @ApiModelProperty("联网状态")
        private Integer netState;

        @ApiModelProperty("商户名称")
        private String mercName;

        @ApiModelProperty("联网类型")
        private String netType;

        @ApiModelProperty("信号强度")
        private Integer netDbm;

        @ApiModelProperty("锁机状态")
        private Integer deviceState;

        @ApiModelProperty("距离")
        private Integer m;
    }
}
